import matplotlib.pyplot as plt
import logomaker
from ..tidy_data.tidy_seqlogoPlot import cleaning
import numpy as np
from ..plots.layout_finder import best_layout
from textwrap import wrap


class LogoPlot:
    def __init__(self,ax, sequencing_report, region_string, sample, chosen_seq_length, highlight_spec_position, font_settings):
        self.ax = ax
        self.aa_distribution = self.cleaningPlot(sample, sequencing_report, chosen_seq_length, region_string)
       # self.createPlot()
        self.chosen_seq_length = chosen_seq_length
        self.highlight_spec_position = highlight_spec_position
        self.font_settings = font_settings
        self.func_type = {
    "alpha": [0.01, 0.01, 0.01, 1],
    "shade": [0.01, 0.01, 0.01, 1],
    "fade": [0.01, 0.01, 0.01, 1],
            }
        self.button_type = {
        "alpha": "CustomScale",
            "shade": "CustomScale",
            "fade": "CustomScale",
        }
        
        
                
    def cleaningPlot(self, sample, sequencing_report, chosen_seq_length, region_string):
        aa_distribution, sequence_length, length_filtered_seqs = cleaning(sample,
                                                                    sequencing_report,
                                                                    chosen_seq_length,
                                                                    region_string)
        return aa_distribution, sequence_length, length_filtered_seqs
    
    def createPlot(self,shade_below = .5, fade_below = .5, font_name = 'Arial Rounded MT Bold', color_scheme = "skylign_protein", show_spines = False,):

        self.logo_plot = logomaker.Logo(
                            self.aa_distribution,
                            shade_below=shade_below,
                            fade_below=fade_below,
                            font_name=font_name,
                            color_scheme=color_scheme,
                            show_spines=show_spines,
                            ax=self.ax,
                            )
        self.logo_plot.style_xticks(anchor=1,
                    spacing=1,
                    rotation=0)
        

    def add_style(self, highlight_specific_pos):
        original_fontsize = self.font_settings["fontsize"]
        self.ax.set_xlabel("Frequency", **self.font_settings)
        self.ax.set_ylabel("Position on sequence", **self.font_settings)
        self.font_settings["fontsize"] = 22
        plt.title("Logo Plot of " + self.sample + " with sequence length " + str(self.chosen_seq_length), **self.font_settings)
        self.font_settings["fontsize"] = original_fontsize
        labels_true = list(range(0, self.chosen_seq_length))
        numbers_true = list(range(1, self.chosen_seq_length + 1))
        plt.xticks(labels_true, numbers_true)
        if highlight_specific_pos != False:
            self.logo_plot.highlight_position(p=5,
                                        color='gold',
                                        alpha=.5)




def plot_logo_single(ax, sequencing_report, sample, font_settings, highlight_specific_pos, region_string,method = "proportion", chosen_seq_length = 16):
    aa_distribution, chosen_seq_length, length_filtered_seqs = cleaning(sample,
                                sequencing_report,
                                chosen_seq_length,
                                region_string,
                                method)

    logo_plot = logomaker.Logo(aa_distribution,
                               shade_below=.5,
                               fade_below=.5,
                               font_name='Arial Rounded MT Bold',
                               color_scheme="skylign_protein",
                               show_spines=False,
                               ax=ax,
                               )
    logo_plot.style_xticks(anchor=1,
                           spacing=1,
                           rotation=0)
    original_fontsize = font_settings["fontsize"]
    if method == "bits":
        ax.set_ylabel('information (bits)',  **font_settings)
        ax.set_xlabel("Position on sequence", **font_settings)
    else:
        ax.set_ylabel("Frequency", **font_settings)
        ax.set_xlabel("Position on sequence", **font_settings)
    font_settings["fontsize"] = 22
   # plt.title("Logo Plot of " + sample + " with sequence length " + str(chosen_seq_length), **font_settings)
    font_settings["fontsize"] = original_fontsize
    labels_true = list(range(0, chosen_seq_length))
    numbers_true = list(range(1, chosen_seq_length + 1))
    plt.xticks(labels_true, numbers_true)
    title = "\n".join(wrap(sample, 40))
    plt.title(title,
            pad=12,
            **font_settings)
    if highlight_specific_pos != False:
        logo_plot.highlight_position(p=5,
                                     color='gold',
                                     alpha=.5)
 #   if highlight_pos_range != False:
  #      logo_plot.ax.highlight_position_range(pmin=3,
   #                                           pmax=5,
    #                                          color="lightcyan")


def plot_logo_multi(fig, sequencing_report, samples, font_settings,region_string,method, chosen_seq_length = 16,):
    if samples == "all":
        unique_experiments = sequencing_report["Experiment"].unique()
        unique_experiments = np.sort(unique_experiments)
    else:
        unique_experiments = sequencing_report["Experiment"].unique()
        unique_experiments = np.array([i for i in unique_experiments if i in samples])
        unique_experiments = np.sort(unique_experiments)
    Tot = unique_experiments.shape[0]
    Rows, Cols = best_layout(Tot)
    Position = range(1, Tot + 1)
    n = 0
    adapted_fontsize = 10 - int(Cols) + 2
    original_fontsize = font_settings["fontsize"]
    font_settings["fontsize"] = adapted_fontsize
  #  fig = plt.figure(1, constrained_layout=True)
    for i in unique_experiments:
        aa_distribution, sequence_length, length_filtered_seqs = cleaning(i,
                                                                          sequencing_report,
                                                                          chosen_seq_length,
                                                                          region_string,
                                                                          method)
        if length_filtered_seqs != 0:
            if length_filtered_seqs < 100:
                print("only " + str(length_filtered_seqs) + " sequences with the given length were found. The results might be biased")
            ax = fig.add_subplot(Rows,
                                 Cols,
                                 Position[n],
                                 xticks = (np.arange(0, chosen_seq_length, step = 1)))
            logo_plot = logomaker.Logo(aa_distribution,
                                        shade_below=.5,
                                        fade_below=.5,
                                        font_name='Arial Rounded MT Bold',
                                        color_scheme="skylign_protein",
                                        show_spines=False,
                                        ax=ax,
                                        )
            #logo_plot.set_xticks(range(aa_distribution.shape[0]))
            logo_plot.style_xticks(anchor=0,
                                   spacing=1,
                                   rotation=0,)
            plt.title(i, **font_settings) # check out https://matplotlib.org/3.1.0/api/_as_gen/matplotlib.pyplot.title.html


            n = n + 1
        else:
            print("Sample " + i + "was skipped because no sequence was found")
    original_fontsize = font_settings["fontsize"]
    font_settings["fontsize"] = 22
    fig.suptitle("Logo Plots for sequence Length " + str(chosen_seq_length), **font_settings)
    font_settings["fontsize"] = original_fontsize





import matplotlib.pyplot as plt
from . import plot_styler
import os

class MyFigure:
    def __init__(self):
        self.fig = plt.figure(1, figsize = (12, 10))
        self.ax = self.fig.gca()
        self.plot_type = "multi"

    def check_fig(self, ):
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)

    def clear_fig(self):
        self.fig.clear()
        if self.plot_type == "single":
            self.ax = self.fig.gca()

    def update_plot(self):
        self.ax = self.fig.gca()
        self.style = plot_styler.PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()
        self.tighten()

    def tighten(self):
        plt.tight_layout()


def save_matrix(matrix, path = None):
    if path == None:
        while True:
            save_matrix = input("Do you want to save the generated data? (Y/n)")
            if save_matrix in ["Y", "y", "N", "n"]:
                break
            else:
                print("Please enter Y or n")
        if save_matrix.lower() in ["Y", "y"]:
            while True:
                filename_matrix = input("Enter a name for the file. The file will be saved locally in your IDE.")
                if not os.path.isfile(filename_matrix):
                    matrix.to_excel(path + ".xlsx")
                    break
                else:
                    print("This file already exists. Please choose another name.")
    else:
        matrix.to_excel(path)





import pandas as pd


def cleaning(report, summed_clonefraction, max_num_reads):
    grouped = report.groupby('Experiment')

    # Step 2: Define a function to get the top rows adding up to 10%
    def get_top_10_percent(group):
        sorted_group = group.sort_values(by='cloneFraction', ascending=False)
        cumulative_sum = sorted_group['cloneFraction'].cumsum()
        return sorted_group[cumulative_sum <= summed_clonefraction][:max_num_reads]

    result_df = grouped.apply(get_top_10_percent)

    # Step 4: Reset index (if needed)
    result_df = result_df.reset_index(drop=True)
    report = result_df
    return report




import editdistance
from ..tidy_data.tidy_sample_cluster import cleaning
import networkx as nx
import math
import seaborn as sns
import community
import matplotlib.pyplot as plt


class ClusterExperiment():
    def __init__(self, sequencing_report, ax, region_string, summed_clonefraction, max_num_reads, edge_color = "peachpuff") -> None:
        report = sequencing_report
        self.region_string = region_string
        report = cleaning(report, summed_clonefraction=summed_clonefraction, max_num_reads=max_num_reads)
        self.G = nx.Graph()
        distances = self.nodes_and_edges(report)
        nodesize = self.customizations(report)
        sample_colors, node_colors = self.create_colors(report)
        self.draw_plot(nodesize, node_colors, ax, edge_color)
        self.legend(report, sample_colors)
        
    def nodes_and_edges(self, report):
        distances = []
        for sample, group in report.groupby('Experiment'):
            sequences = group[self.region_string].tolist()
            for i in range(len(sequences)):
                seq1 = sequences[i]
                # Add node for seq1
                self.G.add_node(seq1, sample=sample)

                for j in range(i + 1, len(sequences)):
                    seq2 = sequences[j]
                    distance = editdistance.distance(seq1, seq2)


                    self.G.add_node(seq2, sample=sample)
                    distances.append((sample, seq1, seq2, distance))
                    # Add nodes for seq2 and connect them with an edge weighted by the Levenshtein distance
                    if distance != 0:
                        self.G.add_edge(seq1, seq2, weight = 1/distance)
                    else:
                        self.G.add_edge(seq1, seq2, weight= 30)
        return distances
    
    def customizations(self, report):
        nodesize = []
        label_numbers = {}
        label_sequences = {}


        for index, g in enumerate(self.G):
            for i in report[self.region_string]:
                if i == g:
                    node = math.sqrt(report.loc[report[self.region_string] == i, "cloneFraction"].values[0])
                    nodesize.append(node)
                    if node * 500 > 1 / 100 * 500:
                        label_numbers[g] = index
                        label_sequences[g] = g
                    else:
                        label_numbers[g] = ""
                        label_sequences[g] = ""
                    break
        for y, n in enumerate(nodesize):
            nodesize[y] = int(n * 500)
        max_fraction_node = max(label_numbers, key=label_numbers.get)
     #   label_numbers = {node: index if node == max_fraction_node else "" for node, index in label_numbers.items()}
      #  label_sequences = {node: seq if node == max_fraction_node else "" for node, seq in label_sequences.items()}

        return nodesize

    def create_colors(self, report):
        num_samples = len(report['Experiment'].unique())

        # Generate a color palette with num_samples distinct colors
        palette = sns.color_palette("hsv", num_samples)

        # Create a dictionary to map samples to colors
        sample_colors = {sample: color for sample, color in zip(report['Experiment'].unique(), palette)}
        node_colors = [sample_colors[self.G.nodes[n]['sample']] for n in self.G.nodes]
        return sample_colors, node_colors
        
    def draw_plot(self, nodesize, node_colors, ax, edge_color):
        G = self.G
        partition = community.best_partition(G)

        weight = 0.1
        
        pos = nx.spring_layout(G)
        nx.draw_networkx_nodes(self.G, pos, node_size=nodesize, node_color = node_colors, ax = ax)
        nx.draw_networkx_edges(self.G, pos, alpha=0.1, edge_color=edge_color, ax = ax)
        
    def legend(self, report, sample_colors):
        unique_samples = report['Experiment'].unique()

                # Create a list of colors corresponding to the samples
        colors = [sample_colors[sample] for sample in unique_samples]

        # Create a legend mapping sample names to colors
        legend = {sample: color for sample, color in zip(unique_samples, colors)}

        # Add a legend to the plot
        plt.legend(handles=[plt.Line2D([0], [0], marker='o', color='w', label=sample,
                                    markersize=10, markerfacecolor=color)
                            for sample, color in legend.items()],
                loc =  'center left', bbox_to_anchor =  (1, 0.5))



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name = "ExpoSeq",
    version = "4.2.2",
    description = "A pacakge which provides various ways to analyze NGS data from phage display campaigns",
    long_description=long_description,
    long_description_content_type = "text/markdown",
    url = "https://github.com/nilshof01/ExpoSeq",
    author = "Nils Hofmann",
    author_email = "nilhof01@gmail.com",
    license = "LICENCE",
    packages = find_packages(where = "src"),
    package_data={
        "ExpoSeq": ["settings/*","settings/*" "test_data/test_files/*", "test_data/*"]
    },
    exclude_package_data={
        '': ['ExpoSeq/test_data/test_files/Chris_main_df.csv']
    },
    install_requires = [
                        "numpy==1.23.5",
                        "pandas==1.5.3",
                        "matplotlib==3.7.3",
                        "scipy>=1.11.3",
                        "seaborn==0.12.2",
                        "logomaker==0.8",
                        "editdistance==0.6.2",
                        "networkx==3.1",
                        "PyQt5==5.15.7",
                        "scikit-learn>=1.2.1",
                        "biopython==1.80",
                        "sgt==2.0.3",
                        "openpyxl",
                        "python-louvain==0.16",
                        "pandasai==1.3.3",
                        "markdown",
                        "sweetviz==2.2.1",
                        "ydata-profiling==4.6.0"
                        ],
    python_requires=">=3.8",
    package_dir = {"": "src"},

)
